@@ -7,11 +7,12 @@ use Import::Into ();
 use MooX::Press 0.025 ();
 use MooX::Press::Keywords ();
 use Syntax::Keyword::Try ();
+use feature ();
 
 package MooX::Pression;
 
 our $AUTHORITY = 'cpan:TOBYINK';
-our $VERSION   = '0.009';
+our $VERSION   = '0.010';
 
 use Keyword::Declare;
 use B::Hooks::EndOfScope;
@@ -334,7 +335,7 @@ my $handle_role_list = sub {
 };
 
 sub _handle_factory_keyword {
-	my ($me, $name, $via, $code, $sig) = @_;
+	my ($me, $name, $via, $code, $sig, $optim) = @_;
 	if ($via) {
 		return sprintf(
 			'q[%s]->_factory(%s, \\(%s));',
@@ -344,27 +345,84 @@ sub _handle_factory_keyword {
 		);
 	}
 	if (!$sig) {
+		my $munged_code = sprintf('sub { my ($factory, $class) = (@_); do %s }', $code);
 		return sprintf(
-			'q[%s]->_factory(%s, sub { my ($factory, $class) = (@_); do %s });',
+			'q[%s]->_factory(%s, { caller => __PACKAGE__, code => %s, optimize => %d });',
 			$me,
 			($name =~ /^\{/ ? "scalar(do $name)" : B::perlstring($name)),
-			$code,
+			$optim ? B::perlstring($munged_code) : $munged_code,
+			!!$optim,
 		);
 	}
 	my ($signature_is_named, $signature_var_list, $type_params_stuff, $extra) = $handle_signature_list->($sig);
 	my $munged_code = sprintf('sub { my($factory,$class,%s)=(shift,shift,@_); %s; do %s }', $signature_var_list, $extra, $code);
 	sprintf(
-		'q[%s]->_factory(%s, { code => %s, named => %d, signature => %s });',
+		'q[%s]->_factory(%s, { caller => __PACKAGE__, code => %s, named => %d, signature => %s, optimize => %d });',
 		$me,
 		($name =~ /^\{/ ? "scalar(do $name)" : B::perlstring($name)),
-		$munged_code,
+		$optim ? B::perlstring($munged_code) : $munged_code,
 		!!$signature_is_named,
 		$type_params_stuff,
+		!!$optim,
 	);
 }
 
+sub _handle_method_keyword {
+	my $me = shift;
+	my ($name, $code, $sig, $optim) = @_;
+	
+	if (defined $name) {
+		if ($sig) {
+			my ($signature_is_named, $signature_var_list, $type_params_stuff, $extra) = $handle_signature_list->($sig);
+			my $munged_code = sprintf('sub { my($self,%s)=(shift,@_); %s; my $class = ref($self)||$self; do %s }', $signature_var_list, $extra, $code);
+			return sprintf(
+				'q[%s]->_can(%s, { caller => __PACKAGE__, code => %s, named => %d, signature => %s, optimize => %d });',
+				$me,
+				($name =~ /^\{/ ? "scalar(do $name)" : B::perlstring($name)),
+				$optim ? B::perlstring($munged_code) : $munged_code,
+				!!$signature_is_named,
+				$type_params_stuff,
+				!!$optim,
+			);
+		}
+		else {
+			my $munged_code = sprintf('sub { my $self = $_[0]; my $class = ref($self)||$self; do %s }', $code);
+			return sprintf(
+				'q[%s]->_can(%s, { caller => __PACKAGE__, code => %s, optimize => %d });',
+				$me,
+				($name =~ /^\{/ ? "scalar(do $name)" : B::perlstring($name)),
+				$optim ? B::perlstring($munged_code) : $munged_code,
+				!!$optim,
+			);
+		}
+	}
+	else {
+		if ($sig) {
+			my ($signature_is_named, $signature_var_list, $type_params_stuff, $extra) = $handle_signature_list->($sig);
+			my $munged_code = sprintf('sub { my($self,%s)=(shift,@_); %s; my $class = ref($self)||$self; do %s }', $signature_var_list, $extra, $code);
+			return sprintf(
+				'q[%s]->wrap_coderef({ caller => __PACKAGE__, code => %s, named => %d, signature => %s, optimize => %d });',
+				'MooX::Press',
+				$optim ? B::perlstring($munged_code) : $munged_code,
+				!!$signature_is_named,
+				$type_params_stuff,
+				!!$optim,
+			);
+		}
+		else {
+			my $munged_code = sprintf('sub { my $self = $_[0]; my $class = ref($self)||$self; do %s }', $code);
+			return sprintf(
+				'q[%s]->wrap_coderef({ caller => __PACKAGE__, code => %s, optimize => %d });',
+				'MooX::Press',
+				$optim ? B::perlstring($munged_code) : $munged_code,
+				!!$optim,
+			);
+		}
+	}
+}
+
 sub _handle_modifier_keyword {
-	my ($me, $kind, $name, $code, $sig) = @_;
+	my ($me, $kind, $name, $code, $sig, $optim) = @_;
 	
 	if ($sig) {
 		my ($signature_is_named, $signature_var_list, $type_params_stuff, $extra) = $handle_signature_list->($sig);
@@ -376,31 +434,36 @@ sub _handle_modifier_keyword {
 			$munged_code = sprintf('sub { my($self,%s)=(shift,@_); %s; my $class = ref($self)||$self; do %s }', $signature_var_list, $extra, $code);
 		}
 		sprintf(
-			'q[%s]->_modifier(q(%s), %s, { code => %s, named => %d, signature => %s });',
+			'q[%s]->_modifier(q(%s), %s, { caller => __PACKAGE__, code => %s, named => %d, signature => %s, optimize => %d });',
 			$me,
 			$kind,
 			($name =~ /^\{/ ? "scalar(do $name)" : B::perlstring($name)),
-			$munged_code,
+			$optim ? B::perlstring($munged_code) : $munged_code,
 			!!$signature_is_named,
 			$type_params_stuff,
+			!!$optim,
 		);
 	}
 	elsif ($kind eq 'around') {
+		my $munged_code = sprintf('sub { my ($next, $self) = @_; my $class = ref($self)||$self; do %s }', $code);
 		sprintf(
-			'q[%s]->_modifier(q(%s), %s, sub { my ($next, $self) = @_; my $class = ref($self)||$self; do %s });',
+			'q[%s]->_modifier(q(%s), %s, { caller => __PACKAGE__, code => %s, optimize => %d });',
 			$me,
 			$kind,
 			($name =~ /^\{/ ? "scalar(do $name)" : B::perlstring($name)),
-			$code,
+			$optim ? B::perlstring($munged_code) : $munged_code,
+			!!$optim,
 		);
 	}
 	else {
+		my $munged_code = sprintf('sub { my $self = $_[0]; my $class = ref($self)||$self; do %s }', $code);
 		sprintf(
-			'q[%s]->_modifier(q(%s), %s, sub { my $self = $_[0]; my $class = ref($self)||$self; do %s });',
+			'q[%s]->_modifier(q(%s), %s, { caller => __PACKAGE__, code => %s, optimize => %d });',
 			$me,
 			$kind,
 			($name =~ /^\{/ ? "scalar(do $name)" : B::perlstring($name)),
-			$code,
+			$optim ? B::perlstring($munged_code) : $munged_code,
+			!!$optim,
 		);
 	}
 }
@@ -412,19 +475,21 @@ sub _handle_modifier_keyword {
 
 sub import {
 	no warnings 'closure';
-	my $caller = caller;
 	my ($me, %opts) = (shift, @_);
+	my $caller = ($opts{caller} ||= caller);
+	
+	# Need to reproduce this logic from MooX::Press to find out
+	# the name of the type library.
+	#
+	require MooX::Press;
+	$opts{prefix}           = $opts{caller} unless exists $opts{prefix};
+	$opts{factory_package}  = $opts{prefix} unless exists $opts{factory_package};
+	$opts{type_library}     = 'Types'       unless exists $opts{type_library};
+	$opts{type_library}     = 'MooX::Press'->qualify_name($opts{type_library}, $opts{prefix});
 	
 	# Optionally export wrapper subs for pre-declared types
 	#
 	if ($opts{declare}) {
-		require MooX::Press;
-		# Need to reproduce this logic from MooX::Press to find out
-		# the name of the type library.
-		$opts{caller}  ||= $caller;
-		$opts{prefix}       = $opts{caller} unless exists $opts{prefix};
-		$opts{type_library} = 'Types'       unless exists $opts{type_library};
-		$opts{type_library} = 'MooX::Press'->qualify_name($opts{type_library}, $opts{prefix});
 		my $types = $opts{type_library};
 		for my $name (@{ $opts{declare} }) {
 			eval qq{
@@ -439,9 +504,14 @@ sub import {
 	# Export utility stuff
 	#
 	MooX::Pression::_Gather->import::into($caller, -gather => %opts);
-	MooX::Press::Keywords->import::into($caller, qw( -booleans -privacy -util ));
+	MooX::Press::Keywords->import::into($caller, qw( -booleans -privacy -util )); # imports strict and warnings
 	Syntax::Keyword::Try->import::into($caller);
-	
+	if ($] >= 5.018) {
+		feature->import::into($caller, qw( say state unicode_strings unicode_eval evalbytes current_sub fc ));
+	}
+	elsif ($] >= 5.014) {
+		feature->import::into($caller, qw( say state unicode_strings ));
+	}
 	$_->import::into($caller, qw( -types -is -assert ))
 		for qw(Types::Standard Types::Common::Numeric Types::Common::String);
 	
@@ -474,6 +544,33 @@ sub import {
 			B::perlstring("class:$plus$classname"),
 		);
 	}
+	keyword class ('(', SignatureList $sig, ')', Block $classdfn) {
+		my ($signature_is_named, $signature_var_list, $type_params_stuff, $extra) = $handle_signature_list->($sig);
+		my $munged_code = sprintf('sub { q(%s)->_package_callback(sub { my ($generator,%s)=(shift,@_); %s; do %s }, @_) }', $me, $signature_var_list, $extra, $classdfn);
+		sprintf(
+			'q[%s]->anonymous_generator(class => { code => %s, named => %d, signature => %s }, toolkit => %s, prefix => %s, factory_package => %s, type_library => %s)',
+			$me,
+			$munged_code,
+			!!$signature_is_named,
+			$type_params_stuff,
+			B::perlstring($opts{toolkit}||'Moo'),
+			B::perlstring($opts{prefix}),
+			B::perlstring($opts{factory_package}),
+			B::perlstring($opts{type_library}),
+		);
+	}
+	keyword class (Block? $classdfn) {
+		$classdfn ||= '{}';
+		sprintf(
+			'q[%s]->anonymous_package(class => sub { do %s }, toolkit => %s, prefix => %s, factory_package => %s, type_library => %s)',
+			$me,
+			$classdfn,
+			B::perlstring($opts{toolkit}||'Moo'),
+			B::perlstring($opts{prefix}),
+			B::perlstring($opts{factory_package}),
+			B::perlstring($opts{type_library}),
+		);
+	}
 	
 	# `role` keyword
 	#
@@ -504,6 +601,33 @@ sub import {
 			B::perlstring('role:'.$classname),
 		);
 	}
+	keyword role ('(', SignatureList $sig, ')', Block $classdfn) {
+		my ($signature_is_named, $signature_var_list, $type_params_stuff, $extra) = $handle_signature_list->($sig);
+		my $munged_code = sprintf('sub { q(%s)->_package_callback(sub { my ($generator,%s)=(shift,@_); %s; do %s }, @_) }', $me, $signature_var_list, $extra, $classdfn);
+		sprintf(
+			'q[%s]->anonymous_generator(role => { code => %s, named => %d, signature => %s }, toolkit => %s, prefix => %s, factory_package => %s, type_library => %s)',
+			$me,
+			$munged_code,
+			!!$signature_is_named,
+			$type_params_stuff,
+			B::perlstring($opts{toolkit}||'Moo'),
+			B::perlstring($opts{prefix}),
+			B::perlstring($opts{factory_package}),
+			B::perlstring($opts{type_library}),
+		);
+	}
+	keyword role (Block? $classdfn) {
+		$classdfn ||= '{}';
+		sprintf(
+			'q[%s]->anonymous_package(role => sub { do %s }, toolkit => %s, prefix => %s, factory_package => %s, type_library => %s)',
+			$me,
+			$classdfn,
+			B::perlstring($opts{toolkit}||'Moo'),
+			B::perlstring($opts{prefix}),
+			B::perlstring($opts{factory_package}),
+			B::perlstring($opts{type_library}),
+		);
+	}
 	
 	# `toolkit` keyword
 	#
@@ -589,67 +713,60 @@ sub import {
 	}
 	
 	# `constant` keyword
+	#
 	keyword constant (Identifier $name, '=', Expr $value) {
 		sprintf('q[%s]->_constant(%s, %s);', $me, B::perlstring($name), $value);
 	}
 	
 	# `method` keyword
 	#
-	keyword method (Identifier|Block $name, '(', SignatureList $sig, ')', Block $code) {
-		my ($signature_is_named, $signature_var_list, $type_params_stuff, $extra) = $handle_signature_list->($sig);
-		my $munged_code = sprintf('sub { my($self,%s)=(shift,@_); %s; my $class = ref($self)||$self; do %s }', $signature_var_list, $extra, $code);
-		sprintf(
-			'q[%s]->_can(%s, { code => %s, named => %d, signature => %s });',
-			$me,
-			($name =~ /^\{/ ? "scalar(do $name)" : B::perlstring($name)),
-			$munged_code,
-			!!$signature_is_named,
-			$type_params_stuff,
-		);
+	keyword method (Identifier|Block $name, ':optimize'? $optim, '(', SignatureList $sig, ')', Block $code) {
+		$me->_handle_method_keyword($name, $code, $sig, !!$optim);
 	}
-	keyword method (Identifier|Block $name, Block $code) {
-		sprintf(
-			'q[%s]->_can(%s, sub { my $self = $_[0]; my $class = ref($self)||$self; do %s });',
-			$me,
-			($name =~ /^\{/ ? "scalar(do $name)" : B::perlstring($name)),
-			$code,
-		);
+	keyword method (Identifier|Block $name, ':optimize'? $optim, Block $code) {
+		$me->_handle_method_keyword($name, $code, undef, !!$optim);
+	}
+	keyword method (                        ':optimize'? $optim, '(', SignatureList $sig, ')', Block $code) {
+		$me->_handle_method_keyword(undef, $code, $sig, !!$optim);
+	}
+	keyword method (                        ':optimize'? $optim, Block $code) {
+		$me->_handle_method_keyword(undef, $code, undef, !!$optim);
 	}
 	
 	# `before`, `after`, and `around` keywords
 	#
-	keyword before (Identifier|Block $name, '(', SignatureList $sig, ')', Block $code) {
-		$me->_handle_modifier_keyword(before => $name, $code, $sig);
+	keyword before (Identifier|Block $name, ':optimize'? $optim, '(', SignatureList $sig, ')', Block $code) {
+		$me->_handle_modifier_keyword(before => $name, $code, $sig, !!$optim);
 	}
-	keyword before (Identifier|Block $name, Block $code) {
-		$me->_handle_modifier_keyword(before => $name, $code);
+	keyword before (Identifier|Block $name, ':optimize'? $optim, Block $code) {
+		$me->_handle_modifier_keyword(before => $name, $code, undef, !!$optim);
 	}
-	keyword after (Identifier|Block $name, '(', SignatureList $sig, ')', Block $code) {
-		$me->_handle_modifier_keyword(after => $name, $code, $sig);
+	keyword after (Identifier|Block $name, ':optimize'? $optim, '(', SignatureList $sig, ')', Block $code) {
+		$me->_handle_modifier_keyword(after => $name, $code, $sig, !!$optim);
 	}
-	keyword after (Identifier|Block $name, Block $code) {
-		$me->_handle_modifier_keyword(after => $name, $code);
+	keyword after (Identifier|Block $name, ':optimize'? $optim, Block $code) {
+		$me->_handle_modifier_keyword(after => $name, $code, undef, !!$optim);
 	}
-	keyword around (Identifier|Block $name, '(', SignatureList $sig, ')', Block $code) {
-		$me->_handle_modifier_keyword(around => $name, $code, $sig);
+	keyword around (Identifier|Block $name, ':optimize'? $optim, '(', SignatureList $sig, ')', Block $code) {
+		$me->_handle_modifier_keyword(around => $name, $code, $sig, !!$optim);
 	}
-	keyword around (Identifier|Block $name, Block $code) {
-		$me->_handle_modifier_keyword(around => $name, $code);
+	keyword around (Identifier|Block $name, ':optimize'? $optim, Block $code) {
+		$me->_handle_modifier_keyword(around => $name, $code, undef, !!$optim);
 	}
 	
 	# `factory` keyword
 	#
-	keyword factory (Identifier|Block $name, '(', SignatureList $sig, ')', Block $code) {
-		$me->_handle_factory_keyword($name, undef, $code, $sig);
+	keyword factory (Identifier|Block $name, ':optimize'? $optim, '(', SignatureList $sig, ')', Block $code) {
+		$me->_handle_factory_keyword($name, undef, $code, $sig, !!$optim);
 	}
-	keyword factory (Identifier|Block $name, Block $code) {
-		$me->_handle_factory_keyword($name, undef, $code, undef);
+	keyword factory (Identifier|Block $name, ':optimize'? $optim, Block $code) {
+		$me->_handle_factory_keyword($name, undef, $code, undef, !!$optim);
 	}
 	keyword factory (Identifier|Block $name, 'via', Identifier $via) {
-		$me->_handle_factory_keyword($name, $via, undef, undef);
+		$me->_handle_factory_keyword($name, $via, undef, undef, !!0);
 	}
 	keyword factory (Identifier|Block $name) {
-		$me->_handle_factory_keyword($name, 'new', undef, undef);
+		$me->_handle_factory_keyword($name, 'new', undef, undef, !!0);
 	}
 	
 	# `coerce` keyword
@@ -768,6 +885,104 @@ sub _modifier {
 	push @{ $OPTS{$kind} ||= [] }, $name, $value;
 }
 
+#{
+#	package MooX::Pression::Anonymous::Package;
+#	our $AUTHORITY = 'cpan:TOBYINK';
+#	our $VERSION   = '0.010';
+#	use overload q[""] => sub { ${$_[0]} }, fallback => 1;
+#	sub DESTROY {}
+#	sub AUTOLOAD {
+#		my $me = shift;
+#		(my $method = our $AUTOLOAD) =~ s/.*:://;
+#		$$me->$method(@_);
+#	}
+#	
+#	package MooX::Pression::Anonymous::Class;
+#	our $AUTHORITY = 'cpan:TOBYINK';
+#	our $VERSION   = '0.010';
+#	our @ISA       = qw(MooX::Pression::Anonymous::Package);
+#	sub new {
+#		my $me = shift;
+#		$$me->new(@_);
+#	}
+#	use overload q[&{}] => sub {
+#		my $me = shift;
+#		sub { $me->new(@_) }
+#	};
+#	
+#	package MooX::Pression::Anonymous::Role;
+#	our $AUTHORITY = 'cpan:TOBYINK';
+#	our $VERSION   = '0.010';
+#	our @ISA       = qw(MooX::Pression::Anonymous::Package);
+#	
+#	package MooX::Pression::Anonymous::ParameterizableClass;
+#	our $AUTHORITY = 'cpan:TOBYINK';
+#	our $VERSION   = '0.010';
+#	our @ISA       = qw(MooX::Pression::Anonymous::Package);
+#	sub generate_package {
+#		my $me  = shift;
+#		my $gen = $$me->generate_package(@_);
+#		bless \$gen, 'MooX::Pression::Anonymous::Class';
+#	}
+#	use overload q[&{}] => sub {
+#		my $me = shift;
+#		sub { $me->new_class(@_) }
+#	};
+#
+#	package MooX::Pression::Anonymous::ParameterizableRole;
+#	our $AUTHORITY = 'cpan:TOBYINK';
+#	our $VERSION   = '0.010';
+#	our @ISA       = qw(MooX::Pression::Anonymous::Package);
+#	sub generate_package {
+#		my $me  = shift;
+#		my $gen = $$me->generate_package(@_);
+#		bless \$gen, 'MooX::Pression::Anonymous::Class';
+#	}
+#	use overload q[&{}] => sub {
+#		my $me = shift;
+#		sub { $me->new_role(@_) }
+#	};
+#}
+
+my $i = 0;
+sub anonymous_package {
+	my $me = shift;
+	my ($kind, $callback, %opts) = @_;
+	my $package_dfn = $me->_package_callback($callback);
+	
+	for my $forbidden (qw/ factory type_name coerce /) {
+		die if exists $package_dfn->{$forbidden};
+	}
+	$package_dfn->{type_name}  = undef;
+	$package_dfn->{factory}    = undef;
+	
+	my $qname = sprintf('%s::__ANON_%06d__', __PACKAGE__, ++$i);
+	
+	require MooX::Press;
+	my $method = "make_$kind";
+	MooX::Press->$method("::$qname", %opts, %$package_dfn);
+	
+	require Module::Runtime;
+	$INC{Module::Runtime::module_notional_filename($qname)} = __FILE__;
+	#return bless(\$qname, "MooX::Pression::Anonymous::".ucfirst($kind));
+	return $qname;
+}
+
+sub anonymous_generator {
+	my $me = shift;
+	my ($kind, $callback, %opts) = @_;
+	my $qname = sprintf('%s::__ANON_%06d__', __PACKAGE__, ++$i);
+	
+	require MooX::Press;
+	my $method = "make_$kind\_generator";
+	MooX::Press->$method("::$qname", %opts, generator => $callback);
+	
+	require Module::Runtime;
+	$INC{Module::Runtime::module_notional_filename($qname)} = __FILE__;
+	#return bless(\$qname, "MooX::Pression::Anonymous::Parameterizable".ucfirst($kind));
+	return $qname;
+}
+
 1;
 
 __END__
@@ -1029,57 +1244,6 @@ It is possible to prefix a class name with a plus sign:
 Now the employee class will be named C<MyApp::Person::Employee> instead of
 the usual C<MyApp::Employee>.
 
-=head4 Parameterizable classes
-
-  package MyApp {
-    use MooX::Pression;
-    
-    class Animal {
-      has name;
-    }
-    
-    class Species ( Str $common_name, Str $binomial ) {
-      extends Animal;
-      constant common_name  = $common_name;
-      constant binomial     = $binomial;
-    }
-    
-    class Dog {
-      extends Species('dog', 'Canis familiaris');
-      method bark () {
-        say "woof!";
-      }
-    }
-  }
-
-Here, "MyApp::Species" isn't a class in the usual sense; you cannot create
-instances of it. It's like a template for generating classes. Then 
-"MyApp::Dog" generates a class from the template and inherits from that.
-
-  my $Cat = MyApp->generate_species('cat', 'Felis catus');
-  my $mog = $Cat->new(name => 'Mog');
-  
-  $mog->isa('MyApp::Animal');         # true
-  $mog->isa('MyApp::Species');        # false!!!
-  $mog->isa($Cat);                    # true
-
-Because there are never any instances of "MyApp::Species", it doesn't
-make sense to have a B<Species> type constraint. Instead there are
-B<SpeciesClass> and B<SpeciesInstance> type constraints.
-
-  use MyApp::Types -is;
-  
-  my $lassie = MyApp->new_dog;
-  
-  is_Animal( $lassie );               # true
-  is_Dog( $lassie );                  # true
-  is_SpeciesInstance( $lassie );      # true
-  is_SpeciesClass( ref($lassie) );    # true
-
-Subclasses cannot be nested inside parameterizable classes.
-It should theoretically be possible to nest parameterizable classes
-within regular classes, but this isn't implemented yet.
-
 =head3 C<< role >>
 
 Define a very basic role:
@@ -1097,31 +1261,6 @@ This is just the same as C<class> but defines a role instead of a class.
 Roles cannot be nested within each other, nor can roles be nested in classes,
 nor classes in roles.
 
-=head4 Parameterizable roles
-
-Often it makes more sense to parameterize roles than classes.
-
-  package MyApp {
-    use MooX::Pression;
-    
-    class Animal {
-      has name;
-    }
-    
-    role Species ( Str $common_name, Str $binomial ) {
-      constant common_name  = $common_name;
-      constant binomial     = $binomial;
-    }
-    
-    class Dog {
-      extends Animal;
-      with Species('dog', 'Canis familiaris'), GoodBoi?;
-      method bark () {
-        say "woof!";
-      }
-    }
-  }
-
 =head3 C<< toolkit >>
 
 Use a different toolkit instead of Moo.
@@ -1604,6 +1743,58 @@ deal with C<< @_ >> in the body of the method. In the second, there
 is a signature, but it is a signature showing that the method expects
 no arguments (other than the invocant of course).
 
+=head4 Optimizing Methods
+
+For a slight compiled-time penalty, you can improve the speed which
+methods run at using the C<< :optimize >> attribute:
+
+  method foo :optimize (...) {
+    ...;
+  }
+
+Optimized methods must not close over any lexical (C<my> or C<our>)
+variables; they can only access the variables declared in their,
+signature, C<< $self >>, C<< $class >>, C<< @_ >>, and globals.
+
+=head4 Anonymous Methods
+
+It I<is> possible to use C<method> without a name to return an
+anonymous method (coderef):
+
+  use MooX::Pression prefix => 'MyApp';
+  
+  class MyClass {
+    method get_method ($foo) {
+      method ($bar) {
+        return $foo . $bar;
+      }
+    }
+  }
+  
+  my $obj   = MyApp->new_myclass;
+  my $anon  = $obj->get_method("FOO");
+  say ref($anon);                       # CODE
+  say $obj->$anon("BAR");               # FOOBAR
+
+Note that while C<< $anon >> is a coderef, it is still a method, and
+still expects to be passed an object as C<< $self >>.
+
+Due to limitations with L<Keyword::Declare> and L<Keyword::Simple>,
+keywords are always complete statements, so C<< method ... >> has an
+implicit semicolon before and after it. This means that this won't
+work:
+
+  my $x = method { ... };
+
+Because it gets treated as:
+
+  my $x = ;
+  method { ... };
+
+A workaround is to wrap it in a C<< do { ... } >> block.
+
+  my $x = do { method { ... } };
+
 =head3 require
 
 Indicates that a role requires classes to fulfil certain methods.
@@ -1648,6 +1839,8 @@ not care about the types of the arguments, so can omit checking them.
     say "Speak now or forever hold your peace!";
   }
 
+The C<< :optimize >> attribute is supported for C<before>.
+
 =head3 C<< after >>
 
 There's not much to say about C<after>. It's just like C<before>.
@@ -1664,6 +1857,8 @@ There's not much to say about C<after>. It's just like C<before>.
     say "You may kiss the bride!";
   }
 
+The C<< :optimize >> attribute is supported for C<after>.
+
 =head3 C<< around >>
 
 The C<around> method modifier is somewhat more interesting.
@@ -1693,6 +1888,8 @@ are not shifted off C<< @_ >> for you, but the variables are still defined.
     return $return;
   }
 
+The C<< :optimize >> attribute is supported for C<around>.
+
 =head3 C<< factory >>
 
 The C<factory> keyword is used to define alternative constructors for
@@ -1793,6 +1990,8 @@ Now C<< MyApp->new_man >> will call C<< MyApp::Person->new_guy >>.
 C<< factory new_person >> with no C<via> or method body is basically
 like saying C<< via new >>.
 
+The C<< :optimize >> attribute is supported for C<factory>.
+
 =head4 Implementing a singleton
 
 Factories make it pretty easy to implement a singleton.
@@ -1874,6 +2073,8 @@ statement more readable.
     $class->new(name => $_);
   }
 
+The C<< :optimize >> attribute is not currently supported for C<coerce>.
+
 =head3 C<< version >>
 
   class Person {
@@ -1931,12 +2132,168 @@ of C<confess> is super-powered and runs its arguments through C<sprintf>.
     }
   }
 
+MooX::Pression turns on strict, warnings, and the following modern Perl
+features:
+
+  # Perl 5.14 and Perl 5.16
+  say state unicode_strings
+  
+  # Perl 5.18 or above
+  say state unicode_strings unicode_eval evalbytes current_sub fc
+
+If you're wondering why not other features, it's because I didn't want to
+enable any that are currently classed as experimental, nor any that require
+a version of Perl above 5.18. The C<switch> feature is mostly seen as a
+failed experiment these days, and C<lexical_subs> cannot be called as methods
+so are less useful in object-oriented code.
+
+You can, of course, turn on extra features yourself.
+
+  package MyApp {
+    use MooX::Pression;
+    use feature qw( lexical_subs postderef );
+    
+    ...;
+  }
+
 And MooX::Pression exports L<Syntax::Keyword::Try> for you. Useful to have.
 
 And last but not least, it exports all the types, C<< is_* >> functions,
 and C<< assert_* >> functions from L<Types::Standard>,
 L<Types::Common::String>, and L<Types::Common::Numeric>.
 
+=head2 Anonymous Classes and Roles
+
+=head3 Anonymous classes
+
+It is possible to make anonymous classes:
+
+  my $class  = do { class; };
+  my $object = $class->new;
+
+The C<< do { ... } >> block is necessary because of a limitation in
+L<Keyword::Declare> and L<Keyword::Simple>, where any keywords they
+define must be complete statements.
+
+Anonymous classes can have methods and attributes and so on:
+
+  my $class = do { class {
+    has foo (type => Int);
+    has bar (type => Int);
+  }};
+  
+  my $object = $class->new(foo => 1, bar => 2);
+
+Anonymous classes I<do not> implicitly inherit from their parent like
+named nested classes do. Named classes nested inside anonymous classes
+I<do not> implicitly inherit from the anonymous class.
+
+Having one anonymous class inherit from another can be done though:
+
+  my $base     = do { class; }
+  my $derived  = do { class {
+    extends {"::$k1"};
+  }};
+
+This works because C<extends> accepts a block which returns a string for
+the package name, and the string needs to begin with "::" to avoid the
+auto prefix mechanism.
+
+=head3 Anonymous roles
+
+Anonymous roles work in much the same way.
+
+=head2 Parameterizable Classes and Roles
+
+=head3 Parameterizable classes
+
+  package MyApp {
+    use MooX::Pression;
+    
+    class Animal {
+      has name;
+    }
+    
+    class Species ( Str $common_name, Str $binomial ) {
+      extends Animal;
+      constant common_name  = $common_name;
+      constant binomial     = $binomial;
+    }
+    
+    class Dog {
+      extends Species('dog', 'Canis familiaris');
+      method bark () {
+        say "woof!";
+      }
+    }
+  }
+
+Here, "MyApp::Species" isn't a class in the usual sense; you cannot create
+instances of it. It's like a template for generating classes. Then 
+"MyApp::Dog" generates a class from the template and inherits from that.
+
+  my $Cat = MyApp->generate_species('cat', 'Felis catus');
+  my $mog = $Cat->new(name => 'Mog');
+  
+  $mog->isa('MyApp::Animal');         # true
+  $mog->isa('MyApp::Species');        # false!!!
+  $mog->isa($Cat);                    # true
+
+Because there are never any instances of "MyApp::Species", it doesn't
+make sense to have a B<Species> type constraint. Instead there are
+B<SpeciesClass> and B<SpeciesInstance> type constraints.
+
+  use MyApp::Types -is;
+  
+  my $lassie = MyApp->new_dog;
+  
+  is_Animal( $lassie );               # true
+  is_Dog( $lassie );                  # true
+  is_SpeciesInstance( $lassie );      # true
+  is_SpeciesClass( ref($lassie) );    # true
+
+Subclasses cannot be nested inside parameterizable classes.
+It should theoretically be possible to nest parameterizable classes
+within regular classes, but this isn't implemented yet.
+
+Anonymous parameterizable classes are possible:
+
+  my $generator = do { class ($footype, $bartype) {
+    has foo (type => $footype);
+    has bar (type => $bartype);
+  };
+  
+  my $class = $generator->generate_package(Int, Num);
+  
+  my $object = $class->new(foo => 42, bar => 4.2);
+
+=head3 Parameterizable roles
+
+Often it makes more sense to parameterize roles than classes.
+
+  package MyApp {
+    use MooX::Pression;
+    
+    class Animal {
+      has name;
+    }
+    
+    role Species ( Str $common_name, Str $binomial ) {
+      constant common_name  = $common_name;
+      constant binomial     = $binomial;
+    }
+    
+    class Dog {
+      extends Animal;
+      with Species('dog', 'Canis familiaris'), GoodBoi?;
+      method bark () {
+        say "woof!";
+      }
+    }
+  }
+
+Anonymous parameterizable roles are possible.
+
 =head2 MooX::Pression vs Moops
 
 MooX::Pression has fewer dependencies than Moops, and crucially doesn't
@@ -1945,6 +2302,8 @@ MooX::Pression uses Damian Conway's excellent L<Keyword::Declare>
 (which in turn uses L<PPR>) to handle most parsing needs, so parsing should
 be more predictable.
 
+Moops is faster though.
+
 Here are a few key syntax and feature differences.
 
 =head3 Declaring a class
@@ -2162,6 +2521,11 @@ MooX::Pression:
     constant PI = 3.2;
   }
 
+=head3 Parameterizable classes and roles
+
+These were always on my todo list for Moops; I doubt they'll ever be done.
+They work nicely in MooX::Pression though.
+
 =head1 BUGS
 
 Please report any bugs to
